@@ -3,7 +3,7 @@
  *  Here we test GiNaC's archiving system. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,16 +20,17 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "exams.h"
-
+#include <iostream>
 #include <fstream>
+#include "ginac.h"
+using namespace std;
+using namespace GiNaC;
 
 unsigned exam_archive()
 {
 	unsigned result = 0;
 	
 	cout << "examining archiving system" << flush;
-	clog << "----------archiving system:" << endl;
 
 	symbol x("x"), y("y"), mu("mu"), dim("dim", "\\Delta");
 	ex e, f;
@@ -70,14 +71,10 @@ unsigned exam_archive()
 		++result;
 	}
 
-	cout << '.' << flush;
-	
-	if (!result) {
-		cout << " passed " << endl;
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed " << endl;
-	}
-	
 	return result;
 }
+
+int main(int argc, char** argv)
+{
+	return exam_archive();
+}
