@@ -3,7 +3,7 @@
  *  Here we test GiNaC's Clifford algebra objects. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,10 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "exams.h"
+#include <iostream>
+#include "ginac.h"
+using namespace std;
+using namespace GiNaC;
 
 const numeric half(1, 2);
 
@@ -539,7 +542,6 @@ unsigned exam_clifford()
 	unsigned result = 0;
 	
 	cout << "examining clifford objects" << flush;
-	clog << "----------clifford objects:" << endl;
 
 	result += clifford_check1(); cout << '.' << flush;
 	result += clifford_check2(); cout << '.' << flush;
@@ -600,12 +602,10 @@ unsigned exam_clifford()
 	result += clifford_check7(indexed(-2*minkmetric(), sy_symm(), xi, chi), dim); cout << '.' << flush;
 	result += clifford_check7(-2*delta_tensor(xi, chi), dim); cout << '.' << flush;
 
-	if (!result) {
-		cout << " passed " << endl;
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed " << endl;
-	}
-
 	return result;
 }
+
+int main(int argc, char** argv)
+{
+	return exam_clifford();
+}
