@@ -4,7 +4,7 @@
  *  rational function normalization in normalization.cpp. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,10 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "exams.h"
+#include <iostream>
+#include "ginac.h"
+using namespace std;
+using namespace GiNaC;
 
 const int MAX_VARIABLES = 3;
 
@@ -231,7 +234,6 @@ unsigned exam_polygcd()
 	unsigned result = 0;
 	
 	cout << "examining polynomial GCD computation" << flush;
-	clog << "----------polynomial GCD computation:" << endl;
 	
 	result += poly_gcd1();  cout << '.' << flush;
 	result += poly_gcd2();  cout << '.' << flush;
@@ -243,12 +245,10 @@ unsigned exam_polygcd()
 	result += poly_gcd6();  cout << '.' << flush;
 	result += poly_gcd7();  cout << '.' << flush;
 	
-	if (!result) {
-		cout << " passed " << endl;
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed " << endl;
-	}
-	
 	return result;
 }
+
+int main(int argc, char** argv)
+{
+	return exam_polygcd();
+}
