@@ -4,7 +4,7 @@
  *  Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,10 +18,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "times.h"
+#include <iostream>
+#include <vector>
+#include "ginac.h"
+#include "timer.h"
+using namespace std;
+using namespace GiNaC;
 
 static unsigned test()
 {
@@ -49,7 +54,6 @@ unsigned time_lw_D()
 	double time = .0;
 	
 	cout << "timing Lewis-Wester test D (normalized sum of rational fcns)" << flush;
-	clog << "-------Lewis-Wester test D (normalized sum of rational fcns):" << endl;
 	
 	rolex.start();
 	// correct for very small times:
@@ -58,14 +62,16 @@ unsigned time_lw_D()
 		++count;
 	} while ((time=rolex.read())<0.1 && !result);
 	cout << '.' << flush;
-	
-	if (!result) {
-		cout << " passed ";
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed ";
-	}
-	cout << int(1000*(time/count))*0.001 << 's' << endl;
+	cout << time/count << 's' << endl;
 	
 	return result;
 }
+
+extern void randomify_symbol_serials();
+
+int main(int argc, char** argv)
+{
+	randomify_symbol_serials();
+	cout << setprecision(2) << showpoint;
+	return time_lw_D();
+}
