@@ -4,7 +4,7 @@
  *  Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "times.h"
+#include <iostream>
+#include <vector>
+#include "ginac.h"
+#include "timer.h"
+using namespace std;
+using namespace GiNaC;
 
 static unsigned test()
 {
@@ -49,7 +54,6 @@ unsigned time_lw_G()
 	double time = .0;
 	
 	cout << "timing Lewis-Wester test G (gcd of 3-var polys)" << flush;
-	clog << "-------Lewis-Wester test G (gcd of 3-var polys):" << endl;
 	
 	rolex.start();
 	// correct for very small times:
@@ -58,14 +62,16 @@ unsigned time_lw_G()
 		++count;
 	} while ((time=rolex.read())<0.1 && !result);
 	cout << '.' << flush;
-	
-	if (!result) {
-		cout << " passed ";
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed ";
-	}
 	cout << time/count << 's' << endl;
 	
 	return result;
 }
+
+extern void randomify_symbol_serials();
+
+int main(int argc, char** argv)
+{
+	randomify_symbol_serials();
+	cout << setprecision(2) << showpoint;
+	return time_lw_G();
+}
