@@ -4,7 +4,7 @@
  *  by Robert H. Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "times.h"
+#include <iostream>
+#include <vector>
+#include "ginac.h"
+#include "timer.h"
+using namespace std;
+using namespace GiNaC;
 
 static unsigned test(unsigned n)
 {
@@ -31,8 +36,6 @@ static unsigned test(unsigned n)
 	
 	cout << "timing Lewis-Wester test " << name
 	     << " (invert rank " << n << " Hilbert)" << flush;
-	clog << "-------Lewis-Wester test " << name
-	     << " (invert rank " << n << " Hilbert):" << endl;
 	
 	// Create a rank n Hilbert matrix:
 	matrix H(n,n);
@@ -44,7 +47,6 @@ static unsigned test(unsigned n)
 	matrix Hinv(n,n);
 	Hinv = H.inverse();
 	cout << ". passed ";
-	clog << "(no output)" << endl;
 	cout << cartier.read() << 's' << endl;
 	
 	// check result:
@@ -52,8 +54,6 @@ static unsigned test(unsigned n)
 	
 	cout << "timing Lewis-Wester test " << name
 	     << " (check rank " << n << " Hilbert)" << flush;
-	clog << "-------Lewis-Wester test " << name
-	     << " (check rank " << n << " Hilbert):" << endl;
 	
 	cartier.reset();
 	matrix identity = H.mul(Hinv);
@@ -68,13 +68,8 @@ static unsigned test(unsigned n)
 					correct = false;
 			}
 		}
-	if (correct) {
-		cout << ". passed ";
-		clog << "(no output)" << endl;
-	} else {
-		cout << ". failed ";
+	if (!correct)
 		++result;
-	}
 	cout << cartier.read() << 's' << endl;
 	return result;
 }
@@ -90,3 +85,12 @@ unsigned time_lw_IJKL()
 	
 	return result;
 }
+
+extern void randomify_symbol_serials();
+
+int main(int argc, char** argv)
+{
+	randomify_symbol_serials();
+	cout << setprecision(2) << showpoint;
+	return time_lw_IJKL();
+}
