@@ -3,7 +3,7 @@
  *  Interface to GiNaC's ABC. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,12 +25,12 @@
 
 #include <cstddef> // for size_t
 #include <vector>
+#include <set>
 #include <map>
 // CINT needs <algorithm> to work properly with <vector>
 #include <algorithm>
 
 #include "flags.h"
-#include "tinfos.h"
 #include "ptr.h"
 #include "assertion.h"
 #include "registrar.h"
@@ -46,9 +46,9 @@ class archive_node;
 class print_context;
 
 typedef std::vector<ex> exvector;
+typedef std::set<ex, ex_is_less> exset;
 typedef std::map<ex, ex, ex_is_less> exmap;
 
-
 // Define this to enable some statistical output for comparisons and hashing
 #undef GINAC_COMPARE_STATISTICS
 
@@ -83,6 +83,7 @@ extern compare_statistics_t compare_statistics;
 
 /** Function object for map(). */
 struct map_function {
+	virtual ~map_function() {}
 	typedef const ex & argument_type;
 	typedef ex result_type;
 	virtual ex operator()(const ex & e) = 0;
@@ -107,7 +108,7 @@ class basic : public refcounted
 	
 	// default constructor, destructor, copy constructor and assignment operator
 protected:
-	basic() : tinfo_key(TINFO_basic), flags(0) {}
+	basic() : tinfo_key(&tinfo_static), flags(0) {}
 
 public:
 	/** basic destructor, virtual because class ex will delete objects of
@@ -122,7 +123,7 @@ public:
 protected:
 	/** Constructor with specified tinfo_key (used by derived classes instead
 	 *  of the default constructor to avoid assigning tinfo_key twice). */
-	basic(unsigned ti) : tinfo_key(ti), flags(0) {}
+	basic(tinfo_t ti) : tinfo_key(ti), flags(0) {}
 	
 	// new virtual functions which can be overridden by derived classes
 public: // only const functions please (may break reference counting)
@@ -161,7 +162,7 @@ public:
 	virtual ex & operator[](size_t i);
 
 	// pattern matching
-	virtual bool has(const ex & other) const;
+	virtual bool has(const ex & other, unsigned options = 0) const;
 	virtual bool match(const ex & pattern, lst & repl_lst) const;
 protected:
 	virtual bool match_same_type(const basic & other) const;
@@ -181,6 +182,7 @@ public:
 	}
 
 	// degree/coeff
+	virtual bool is_polynomial(const ex & var) const;
 	virtual int degree(const ex & s) const;
 	virtual int ldegree(const ex & s) const;
 	virtual ex coeff(const ex & s, int n = 1) const;
@@ -213,10 +215,12 @@ public:
 
 	// noncommutativity
 	virtual unsigned return_type() const;
-	virtual unsigned return_type_tinfo() const;
+	virtual tinfo_t return_type_tinfo() const;
 
-	// complex conjugation
+	// functions for complex expressions
 	virtual ex conjugate() const;
+	virtual ex real_part() const;
+	virtual ex imag_part() const;
 
 	// functions that should be called from class ex only
 protected:
@@ -261,7 +265,7 @@ public:
 		}
 	}
 
-	unsigned tinfo() const {return tinfo_key;}
+	tinfo_t tinfo() const {return tinfo_key;}
 
 	/** Set some status_flags. */
 	const basic & setflag(unsigned f) const {flags |= f; return *this;}
@@ -278,7 +282,7 @@ protected:
 	
 	// member variables
 protected:
-	unsigned tinfo_key;                 ///< typeinfo
+	tinfo_t tinfo_key;                  ///< type info
 	mutable unsigned flags;             ///< of type status_flags
 	mutable unsigned hashvalue;         ///< hash value
 };
@@ -298,13 +302,11 @@ inline bool is_a(const basic &obj)
 	return dynamic_cast<const T *>(&obj) != 0;
 }
 
-/** Check if obj is a T, not including base classes.  This one is just an
- *  inefficient default.  It should in all time-critical cases be overridden
- *  by template specializations that use the TINFO_* constants directly. */
+/** Check if obj is a T, not including base classes. */
 template <class T>
-inline bool is_exactly_a(const basic &obj)
+inline bool is_exactly_a(const basic & obj)
 {
-	return obj.tinfo() == T::get_class_info_static().options.get_id();
+	return obj.tinfo() == &T::tinfo_static;
 }
 
 } // namespace GiNaC
