@@ -4,7 +4,7 @@
  *  some related stuff. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -54,8 +54,7 @@ static ex lgamma_evalf(const ex & x)
 
 
 /** Evaluation of lgamma(x), the natural logarithm of the Gamma function.
- *  Knows about integer arguments and that's it.  Somebody ought to provide
- *  some good numerical evaluation some day...
+ *  Handles integer arguments as a special case.
  *
  *  @exception GiNaC::pole_error("lgamma_eval(): logarithmic pole",0) */
 static ex lgamma_eval(const ex & x)
@@ -69,7 +68,8 @@ static ex lgamma_eval(const ex & x)
 			else
 				throw (pole_error("lgamma_eval(): logarithmic pole",0));
 		}
-		//  lgamma_evalf should be called here once it becomes available
+		if (!ex_to<numeric>(x).is_rational())
+			return lgamma(ex_to<numeric>(x));
 	}
 	
 	return lgamma(x).hold();
@@ -165,7 +165,8 @@ static ex tgamma_eval(const ex & x)
 				return (pow(*_num_2_p, n).div(doublefactorial(n.mul(*_num2_p).sub(*_num1_p))))*sqrt(Pi);
 			}
 		}
-		//  tgamma_evalf should be called here once it becomes available
+		if (!ex_to<numeric>(x).is_rational())
+			return tgamma(ex_to<numeric>(x));
 	}
 	
 	return tgamma(x).hold();
@@ -221,7 +222,7 @@ static ex beta_evalf(const ex & x, const ex & y)
 {
 	if (is_exactly_a<numeric>(x) && is_exactly_a<numeric>(y)) {
 		try {
-			return tgamma(ex_to<numeric>(x))*tgamma(ex_to<numeric>(y))/tgamma(ex_to<numeric>(x+y));
+			return exp(lgamma(ex_to<numeric>(x))+lgamma(ex_to<numeric>(y))-lgamma(ex_to<numeric>(x+y)));
 		} catch (const dunno &e) { }
 	}
 	
@@ -262,7 +263,8 @@ static ex beta_eval(const ex & x, const ex & y)
 		    (nx+ny).is_integer() &&
 		   !(nx+ny).is_positive())
 			 return _ex0;
-		// beta_evalf should be called here once it becomes available
+		if (!ex_to<numeric>(x).is_rational() || !ex_to<numeric>(x).is_rational())
+			return evalf(beta(x, y).hold());
 	}
 	
 	return beta(x,y).hold();
