@@ -3,7 +3,7 @@
  *  Interface to symbolic matrices */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -121,6 +121,8 @@ public:
 	ex scalar_mul_indexed(const ex & self, const numeric & other) const;
 	bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const;
 	ex conjugate() const;
+	ex real_part() const;
+	ex imag_part() const;
 
 protected:
 	bool match_same_type(const basic & other) const;
@@ -149,6 +151,7 @@ public:
 	matrix solve(const matrix & vars, const matrix & rhs,
 	             unsigned algo = solve_algo::automatic) const;
 	unsigned rank() const;
+	bool is_zero_matrix() const;
 protected:
 	ex determinant_minor() const;
 	int gauss_elimination(const bool det = false);
@@ -209,12 +212,6 @@ inline unsigned rank(const matrix & m)
 
 // utility functions
 
-/** Specialization of is_exactly_a<matrix>(obj) for matrix objects. */
-template<> inline bool is_exactly_a<matrix>(const basic & obj)
-{
-	return obj.tinfo()==TINFO_matrix;
-}
-
 /** Convert list of lists to matrix. */
 extern ex lst_to_matrix(const lst & l);
 
