@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's symbolic exponentiation (basis^exponent). */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,6 +40,8 @@
 #include "lst.h"
 #include "archive.h"
 #include "utils.h"
+#include "relational.h"
+#include "compiler.h"
 
 namespace GiNaC {
 
@@ -56,7 +58,7 @@ typedef std::vector<int> intvector;
 // default constructor
 //////////
 
-power::power() : inherited(TINFO_power) { }
+power::power() : inherited(&power::tinfo_static) { }
 
 //////////
 // other constructors
@@ -162,6 +164,21 @@ static void print_sym_pow(const print_context & c, const symbol &x, int exp)
 
 void power::do_print_csrc(const print_csrc & c, unsigned level) const
 {
+	if (is_a<print_csrc_cl_N>(c)) {
+		if (exponent.is_equal(_ex_1)) {
+			c.s << "recip(";
+			basis.print(c);
+			c.s << ')';
+			return;
+		}
+		c.s << "expt(";
+		basis.print(c);
+		c.s << ", ";
+		exponent.print(c);
+		c.s << ')';
+		return;
+	}
+
 	// Integer powers of symbols are printed in a special, optimized way
 	if (exponent.info(info_flags::integer)
 	 && (is_a<symbol>(basis) || is_a<constant>(basis))) {
@@ -170,29 +187,20 @@ void power::do_print_csrc(const print_csrc & c, unsigned level) const
 			c.s << '(';
 		else {
 			exp = -exp;
-			if (is_a<print_csrc_cl_N>(c))
-				c.s << "recip(";
-			else
-				c.s << "1.0/(";
+			c.s << "1.0/(";
 		}
 		print_sym_pow(c, ex_to<symbol>(basis), exp);
 		c.s << ')';
 
 	// <expr>^-1 is printed as "1.0/<expr>" or with the recip() function of CLN
 	} else if (exponent.is_equal(_ex_1)) {
-		if (is_a<print_csrc_cl_N>(c))
-			c.s << "recip(";
-		else
-			c.s << "1.0/(";
+		c.s << "1.0/(";
 		basis.print(c);
 		c.s << ')';
 
-	// Otherwise, use the pow() or expt() (CLN) functions
+	// Otherwise, use the pow() function
 	} else {
-		if (is_a<print_csrc_cl_N>(c))
-			c.s << "expt(";
-		else
-			c.s << "pow(";
+		c.s << "pow(";
 		basis.print(c);
 		c.s << ',';
 		exponent.print(c);
@@ -230,6 +238,23 @@ bool power::info(unsigned inf) const
 		case info_flags::algebraic:
 			return !exponent.info(info_flags::integer) ||
 			       basis.info(inf);
+		case info_flags::expanded:
+			return (flags & status_flags::expanded);
+		case info_flags::has_indices: {
+			if (flags & status_flags::has_indices)
+				return true;
+			else if (flags & status_flags::has_no_indices)
+				return false;
+			else if (basis.info(info_flags::has_indices)) {
+				setflag(status_flags::has_indices);
+				clearflag(status_flags::has_no_indices);
+				return true;
+			} else {
+				clearflag(status_flags::has_indices);
+				setflag(status_flags::has_no_indices);
+				return false;
+			}
+		}
 	}
 	return inherited::info(inf);
 }
@@ -258,6 +283,15 @@ ex power::map(map_function & f) const
 		return *this;
 }
 
+bool power::is_polynomial(const ex & var) const
+{
+	if (exponent.has(var))
+		return false;
+	if (!exponent.info(info_flags::nonnegint))
+		return false;
+	return basis.is_polynomial(var);
+}
+
 int power::degree(const ex & s) const
 {
 	if (is_equal(ex_to<basic>(s)))
@@ -382,6 +416,10 @@ ex power::eval(int level) const
 	if (ebasis.is_equal(_ex1))
 		return _ex1;
 
+	// power of a function calculated by separate rules defined for this function
+	if (is_exactly_a<function>(ebasis))
+		return ex_to<function>(ebasis).power(eexponent);
+
 	// Turn (x^c)^d into x^(c*d) in the case that x is positive and c is real.
 	if (is_exactly_a<power>(ebasis) && ebasis.op(0).info(info_flags::positive) && ebasis.op(1).info(info_flags::real))
 		return power(ebasis.op(0), ebasis.op(1) * eexponent);
@@ -453,8 +491,9 @@ ex power::eval(int level) const
 			if (is_exactly_a<numeric>(sub_exponent)) {
 				const numeric & num_sub_exponent = ex_to<numeric>(sub_exponent);
 				GINAC_ASSERT(num_sub_exponent!=numeric(1));
-				if (num_exponent->is_integer() || (abs(num_sub_exponent) - (*_num1_p)).is_negative())
+				if (num_exponent->is_integer() || (abs(num_sub_exponent) - (*_num1_p)).is_negative()) {
 					return power(sub_basis,num_sub_exponent.mul(*num_exponent));
+				}
 			}
 		}
 	
@@ -462,7 +501,35 @@ ex power::eval(int level) const
 		if (num_exponent->is_integer() && is_exactly_a<mul>(ebasis)) {
 			return expand_mul(ex_to<mul>(ebasis), *num_exponent, 0);
 		}
-	
+
+		// (2*x + 6*y)^(-4) -> 1/16*(x + 3*y)^(-4)
+		if (num_exponent->is_integer() && is_exactly_a<add>(ebasis)) {
+			numeric icont = ebasis.integer_content();
+			const numeric lead_coeff = 
+				ex_to<numeric>(ex_to<add>(ebasis).seq.begin()->coeff).div(icont);
+
+			const bool canonicalizable = lead_coeff.is_integer();
+			const bool unit_normal = lead_coeff.is_pos_integer();
+			if (canonicalizable && (! unit_normal))
+				icont = icont.mul(*_num_1_p);
+			
+			if (canonicalizable && (icont != *_num1_p)) {
+				const add& addref = ex_to<add>(ebasis);
+				add* addp = new add(addref);
+				addp->setflag(status_flags::dynallocated);
+				addp->clearflag(status_flags::hash_calculated);
+				addp->overall_coeff = ex_to<numeric>(addp->overall_coeff).div_dyn(icont);
+				for (epvector::iterator i = addp->seq.begin(); i != addp->seq.end(); ++i)
+					i->coeff = ex_to<numeric>(i->coeff).div_dyn(icont);
+
+				const numeric c = icont.power(*num_exponent);
+				if (likely(c != *_num1_p))
+					return (new mul(power(*addp, *num_exponent), c))->setflag(status_flags::dynallocated);
+				else
+					return power(*addp, *num_exponent);
+			}
+		}
+
 		// ^(*(...,x;c1),c2) -> *(^(*(...,x;1),c2),c1^c2)  (c1, c2 numeric(), c1>0)
 		// ^(*(...,x;c1),c2) -> *(^(*(...,x;-1),c2),(-c1)^c2)  (c1, c2 numeric(), c1<0)
 		if (is_exactly_a<mul>(ebasis)) {
@@ -542,6 +609,30 @@ ex power::evalm() const
 	return (new power(ebasis, eexponent))->setflag(status_flags::dynallocated);
 }
 
+bool power::has(const ex & other, unsigned options) const
+{
+	if (!(options & has_options::algebraic))
+		return basic::has(other, options);
+	if (!is_a<power>(other))
+		return basic::has(other, options);
+	if (!exponent.info(info_flags::integer)
+			|| !other.op(1).info(info_flags::integer))
+		return basic::has(other, options);
+	if (exponent.info(info_flags::posint)
+			&& other.op(1).info(info_flags::posint)
+			&& ex_to<numeric>(exponent).to_int()
+					> ex_to<numeric>(other.op(1)).to_int()
+			&& basis.match(other.op(0)))
+		return true;
+	if (exponent.info(info_flags::negint)
+			&& other.op(1).info(info_flags::negint)
+			&& ex_to<numeric>(exponent).to_int()
+					< ex_to<numeric>(other.op(1)).to_int()
+			&& basis.match(other.op(0)))
+		return true;
+	return basic::has(other, options);
+}
+
 // from mul.cpp
 extern bool tryfactsubs(const ex &, const ex &, int &, lst &);
 
@@ -582,6 +673,59 @@ ex power::conjugate() const
 	return (new power(newbasis, newexponent))->setflag(status_flags::dynallocated);
 }
 
+ex power::real_part() const
+{
+	if (exponent.info(info_flags::integer)) {
+		ex basis_real = basis.real_part();
+		if (basis_real == basis)
+			return *this;
+		realsymbol a("a"),b("b");
+		ex result;
+		if (exponent.info(info_flags::posint))
+			result = power(a+I*b,exponent);
+		else
+			result = power(a/(a*a+b*b)-I*b/(a*a+b*b),-exponent);
+		result = result.expand();
+		result = result.real_part();
+		result = result.subs(lst( a==basis_real, b==basis.imag_part() ));
+		return result;
+	}
+	
+	ex a = basis.real_part();
+	ex b = basis.imag_part();
+	ex c = exponent.real_part();
+	ex d = exponent.imag_part();
+	return power(abs(basis),c)*exp(-d*atan2(b,a))*cos(c*atan2(b,a)+d*log(abs(basis)));
+}
+
+ex power::imag_part() const
+{
+	if (exponent.info(info_flags::integer)) {
+		ex basis_real = basis.real_part();
+		if (basis_real == basis)
+			return 0;
+		realsymbol a("a"),b("b");
+		ex result;
+		if (exponent.info(info_flags::posint))
+			result = power(a+I*b,exponent);
+		else
+			result = power(a/(a*a+b*b)-I*b/(a*a+b*b),-exponent);
+		result = result.expand();
+		result = result.imag_part();
+		result = result.subs(lst( a==basis_real, b==basis.imag_part() ));
+		return result;
+	}
+	
+	ex a=basis.real_part();
+	ex b=basis.imag_part();
+	ex c=exponent.real_part();
+	ex d=exponent.imag_part();
+	return
+		power(abs(basis),c)*exp(-d*atan2(b,a))*sin(c*atan2(b,a)+d*log(abs(basis)));
+}
+
+// protected
+
 // protected
 
 /** Implementation of ex::diff() for a power.
@@ -619,17 +763,20 @@ unsigned power::return_type() const
 {
 	return basis.return_type();
 }
-   
-unsigned power::return_type_tinfo() const
+
+tinfo_t power::return_type_tinfo() const
 {
 	return basis.return_type_tinfo();
 }
 
 ex power::expand(unsigned options) const
 {
-	if (options == 0 && (flags & status_flags::expanded))
+	if (is_a<symbol>(basis) && exponent.info(info_flags::integer)) {
+		// A special case worth optimizing.
+		setflag(status_flags::expanded);
 		return *this;
-	
+	}
+
 	const ex expanded_basis = basis.expand(options);
 	const ex expanded_exponent = exponent.expand(options);
 	
@@ -862,11 +1009,19 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 		return _ex1;
 	}
 
+	// do not bother to rename indices if there are no any.
+	if ((!(options & expand_options::expand_rename_idx)) 
+			&& m.info(info_flags::has_indices))
+		options |= expand_options::expand_rename_idx;
 	// Leave it to multiplication since dummy indices have to be renamed
-	if (get_all_dummy_indices(m).size() > 0 && n.is_positive()) {
+	if ((options & expand_options::expand_rename_idx) &&
+		(get_all_dummy_indices(m).size() > 0) && n.is_positive()) {
 		ex result = m;
+		exvector va = get_all_dummy_indices(m);
+		sort(va.begin(), va.end(), ex_is_less());
+
 		for (int i=1; i < n.to_int(); i++)
-			result *= rename_dummy_indices_uniquely(m,m);
+			result *= rename_dummy_indices_uniquely(va, m);
 		return result;
 	}
 
